@@ -142,7 +142,20 @@ PGAPI_BindParameter(HSTMT hstmt,
 }
 
 
-/*	Associate a user-supplied buffer with a database column. */
+/** 
+ * @brief a user-supplied buffer with a database column. 
+ * @param hstmt		Statement handle
+ * @param icol		Column number (zero-based)
+ * @param fCType	Column data type
+ * @param rgbValue	Column data
+ * @param cbValueMax	Maximum column data size
+ * @param pcbValue	Actual column data size
+ * @return RETCODE
+ * 	SQL_SUCCESS
+ * 	SQL_SUCCESS_WITH_INFO
+ * 	SQL_ERROR
+ * 	SQL_INVALID_HANDLE
+*/
 RETCODE		SQL_API
 PGAPI_BindCol(HSTMT hstmt,
 			  SQLUSMALLINT icol,
@@ -509,7 +522,7 @@ extend_parameter_bindings(APDFields *self, int num_params)
 			self->allocated = 0;
 			return;
 		}
-		memset(&new_bindings[self->allocated], 0, sizeof(ParameterInfoClass) * (num_params - self->allocated));
+		pg_memset(&new_bindings[self->allocated], 0, sizeof(ParameterInfoClass) * (num_params - self->allocated));
 
 		self->parameters = new_bindings;
 		self->allocated = num_params;
@@ -542,7 +555,7 @@ extend_iparameter_bindings(IPDFields *self, int num_params)
 			self->allocated = 0;
 			return;
 		}
-		memset(&new_bindings[self->allocated], 0,
+		pg_memset(&new_bindings[self->allocated], 0,
 			sizeof(ParameterImplClass) * (num_params - self->allocated));
 
 		self->parameters = new_bindings;
@@ -968,7 +981,7 @@ extend_putdata_info(PutDataInfo *self, int num_params, BOOL shrink)
 			self->allocated = 0;
 			return;
 		}
-		memset(&new_pdata[self->allocated], 0,
+		pg_memset(&new_pdata[self->allocated], 0,
 			sizeof(PutDataClass) * (num_params - self->allocated));
 
 		self->pdata = new_pdata;
